/*
 *--- ColorerPlugin.cp ---------------------------------------------
 * Copyright (c) 1995-1996, Adobe Systems, Inc.  All rights reserved.
 * Created on Thu, Oct 12, 1995 @ 9:50 PM by Paul Ferguson.
 *
 * The Colorer plug-in illustrates working with a variety of
 * command and query classes from the PageMaker Class Library.
 *
 * This class does the following:
 * 	1) Saves PageMaker's current state (page number, etc.)
 * 	2) Creates a dialog box for the user to specify
 *     several parameters about what things to color.
 * 	3) Colors chosen PageMaker objects according to
 *     those parameters.
 * 	4) Returns PageMaker to its previous state.
 *
 * The auxiliary class "ColorerDialog" encapsulates platform
 * specific code to implement a dialog box to get user's
 * settings, which are stored in a Utility class, UColorerData.
 *
 * If the user would like to run the plug-in (clicks "OK" in
 * the dialog), the ColorDialog objects sends a
 * ColorerPlugin::ColorObjects() message back to this class.  The
 * ColorObjects() method handles the logic of recoloring
 * the rectangles, ovals, and lines in the current PageMaker
 * document.
 *------------------------------------------------------------------
 */

#include <string.h>

#include "PMPlugin.h"

#include "ColorerPlugin.h"
#include "ColorerDialog.h"

#include "PCL.h"

extern sPMParamBlockPtr gPB;

/*
 *--- ColorerPlugin ------------------------------------------------
 * No special initialization needed.
 *------------------------------------------------------------------
 */
ColorerPlugin::ColorerPlugin() : PPluginCall()
{

}


/*
 *--- DoLoad -------------------------------------------------------
 * Create the colorer dialog data structure and store it in the
 * plug-in's data handle.  This preserves settings between
 * invocations of the plug-in.
 *
 * WINDOWS ONLY- GlobalAlloc is used to allocate a fixed memory space
 * for the Plug-in's data block.
 * If new were used, the memory would be tied to the DLL, which gets 
 * unloaded after each invocation.
 *------------------------------------------------------------------
 */
void ColorerPlugin::DoLoad()
{
	if(GetPluginData() == NULL)
	{
#ifdef MACINTOSH
		UColorerData * cd = new UColorerData;
#else //WINDOWS
		UColorerData * cd = (UColorerData *) GlobalAlloc(GMEM_FIXED,sizeof(UColorerData));
		if (cd == NULL) throw CQ_FAILURE;
		cd->doRectangles = true;			   // since new isn't used
		cd->doOvals = true;					   // the constructor isn't called.
		cd->doLines = true;
		cd->radioChoice = (choiceAllPages);
		cd->firstPage = 0;
		cd->lastPage = 0;
		cd->rangeText[0] = '\0';
		cd->colorText[0] = '\0';
#endif //MACINTOSH/WINDOWS 
		SetPluginData(cd);
	}
			
}

/*
 *--- DoInvoke -----------------------------------------------------
 * Checks to make sure that a PageMaker document is open.  If not,
 * it sets the error text using the BuildErrorMessages() method
 * (from this class' superclass), and throws an exception.
 *------------------------------------------------------------------
 */
void ColorerPlugin::DoInvoke() throw (PMErr)
{
	PGetPMState theState;
	char myBuffer[300];
	char *myBufPtr=myBuffer;
	unsigned long sizeBuf = 0;
	
	sizeBuf += LPPutString(myBufPtr, "C:\\europa\\PRODUCT.PM6\\blah.p65");
	myBufPtr += sizeBuf;
	sizeBuf += LPPutShort(myBufPtr, 1);

	PMErr result = PBBinCommand(gPB, pm_open, kXRSPointer, &myBuffer, sizeBuf);

	if ((short) theState == 1) // check for no open publication
	{
		this->BuildErrorMessages("No publication is open.", "");
		throw CQ_INVALID_STATE;
	}

	ColorerDialog theDialog(*this);
	throw CQ_FAILURE;
}

void ColorerPlugin::DoUnload()
{

}

void ColorerPlugin::DoCleanup()
{
	UColorerData * cd = (UColorerData *) GetPluginData();
	if(cd)
	{
#ifdef MACINTOSH
		delete cd;
#else //WINDOWS
		GlobalFree(cd);
#endif
		SetPluginData(NULL);
	}
}

void ColorerPlugin::DoShutdown()
{
	DoCleanup();		// cleanup and shutdown do the same thing.
}

/*
 *--- ParsePages ---------------------------------------------------
 * Converts the text in variable rangeString into integers firstPage
 * and lastPage, or returns false if this cannot be done properly.
 *
 * This method can be called by the ColorerDialog class to validate
 * input while the user is in the dialog.
 *
 * Note that sscanf cannot be used since this is a DLL.
 * Also, This routine allows for cases such as "4", "-4", "4-",
 * and "4-5", and it checks for errors.
 *------------------------------------------------------------------
 */
PMBool ColorerPlugin::ParsePages(UColorerData &colorData)
{
	short firstPage, lastPage;
	char tempString[256];
	short i, lastValidPage, dashPosition, firstWordPosition;
	PGetPages numPages;			// number of pages in the document

	strcpy(tempString, colorData.rangeText);

	//--- Find dash position and convert to space -----------
	dashPosition = -1;
	i = 0;
	while (tempString[i] != '\0')
	{
		if (tempString[i] == '-')
		{
			if (dashPosition != -1)
				return false; 			// Only one dash allowed!
			else
			{
				dashPosition = i;
				tempString[i] = ' ';
			}
		}
		// Reject any non-legal characters
		if (tempString[i] != ' ' && (tempString[i] < '0' || tempString[i] > '9') )
			return false;
		i++;
	}

	
	//--- Parse first number --------------------------------
	i = 0;
	while (tempString[i] == ' ')
		i++;

	firstWordPosition = i; 			// Remember where the first word started
	if (tempString[i] == '\0')
		return false; 				// No numbers, only spaces

	firstPage = 0;					// Proceed to end of first number
	while (tempString[i] >= '0' && tempString[i] <= '9')
	{
		firstPage = firstPage*10 + (short)(tempString[i]-'0');
		i++;
	}
	
	
	//--- Parse second number ------------------------------
	while (tempString[i] == ' ')
		i++;

	if (tempString[i] == '\0')		// Only one number given
	{
		if (dashPosition == -1) 	// No dash given. (e.g. "4")
			lastPage = firstPage;
		else if (dashPosition < firstWordPosition) // Leading dash (e.g. "-4")
		{
			lastPage = firstPage;
			firstPage = 1;
		}
		else
		{
			lastPage = numPages; 	// Trailing dash (e.g "4-")
		}
	}
	else 			// At least two numbers were given. (e.g. "4-5")
	{				// Proceed to end of second number.
		lastPage = 0;
		while (tempString[i] >= '0' && tempString[i] <= '9')
		{
			lastPage = lastPage * 10 + (short)(tempString[i] - '0');
			i++;
		}
		// Make sure there were not more than two numbers given
		while (tempString[i] == ' ')
			i++;
		if (tempString[i] != '\0') // Egads! A third number is present!
			return false;
	}
	
	//--- Check the two numbers for errors ----------------
	lastValidPage = numPages;

	if (firstPage < 1 || firstPage > lastValidPage)
		return false;

	if (lastPage < 1 || lastPage > lastValidPage)
		return false;

	if (firstPage > lastPage)
		return false;

	colorData.firstPage = firstPage;
	colorData.lastPage  = lastPage;

	return true; 			// The number or numbers are legal
}


/*
 *--- ColorObjects -------------------------------------------------
 * Uses the information in UColorerData to color a subset of all the
 * objects in a PageMaker document.
 *
 * This method should be called from ColorDialog class if the user
 * wants to apply her choices.
 *------------------------------------------------------------------
 */
void ColorerPlugin::ColorObjects() throw (PMErr)
{
	PGetPageNumber		savePage;				// PageMaker query for current page number
	PGetPageOptions	savePageOptions;		// PageMaker query for page options
		
	PPageOptions newPageOptions(false, false);	// This plug-in needs double-sided to be OFF
	
	UColorerData & colorData = *((UColorerData *) this->GetPluginData());
	if (&colorData == NULL)
		throw CQ_FAILURE;
		
	// Fill in the user's choice variables with the plug-in Data values.
 	this->ConvertParameters(colorData, savePageOptions.bDoubleSided);

	PDeselect deselect;						// PageMaker command to deselect anything that is selected
	
	// Loop through each chosen page
	for (short i = colorData.firstPage; i <= colorData.lastPage; i++)
	{
		PPage thePage(i,"\0"); 
		PGetObjectList objectList;				// PageMaker query to get list of the objects on page i.

		short n = objectList.Count();
		

		for (short j = 0; j < n; j++)
		{
			if (   (colorData.doRectangles && objectList.cTypeOfObject == 4 )
				|| (colorData.doOvals      && objectList.cTypeOfObject == 5 )
				|| (colorData.doLines      && objectList.cTypeOfObject == 3 ))
			{
				
				PSelect   select(objectList.nDrawingNumber);		// PageMaker command to select this object
				PColor	  color(colorData.colorText, 1000);		// PageMaker command to set the color of the selected object					
				PDeselect deselect;
			}
			objectList++;
		}
	}
	
	PPageOptions	restorePageOptions(savePageOptions); 	// PageMaker command to restore page setup
	PPage			restorePage(savePage.nPageNumber,(char *)savePage.sMasterName); 					// PageMaker command to restore user's original page
}

/*
 *--- ConvertParameters -------------------------------------------------
 * Retrieves the plug-in data parameters and converts rangeString
 * to firstPage and lastPage.
  *------------------------------------------------------------------
 */
void ColorerPlugin::ConvertParameters(UColorerData &colorData, const PMBool bDoubleSided)
{
	switch (colorData.radioChoice)
	{
	case choiceAllPages:
		{
			colorData.firstPage = 1;
			PGetPages numPages;			// PageMaker query for number of pages
			colorData.lastPage = numPages;
		}
		break;

	case choiceCurrentPage:
		{
			PGetPageNumber currentPage;		// PageMaker query for current page number
			colorData.lastPage = currentPage;
			colorData.firstPage = colorData.lastPage;
			if (colorData.firstPage > 1 && bDoubleSided)
				(colorData.firstPage)--;
		}
		break;
	
	case choicePageRange:
		ParsePages(colorData);		// NOTE: the validity of rangeString should already be verified in the dialog class.
		break;
	}
}

/*
 *--- UColorerData constructor -------------------------------------
 * Initialize our data for the dialog.
 *------------------------------------------------------------------
 */
UColorerData::UColorerData()
: doRectangles(true),
  doOvals(true),
  doLines(true),
  radioChoice(choiceAllPages),
  firstPage(0),
  lastPage(0)
{
	rangeText[0] = '\0';
	colorText[0] = '\0';


}

// end of Colorer.cp
